/* ----------------------------------------------------------------------------
 * rishu_proxy_request.cpp
 *   里珠/Proxy の要求の実装.
 * ----------------------------------------------------------------------------
 * Mastering programed by YAMASHINA Hio
 *
 * Copyright 2002 YMIRLINK,Inc.
 * ----------------------------------------------------------------------------
 * $Id: rishu_proxy_request.c,v 1.6 2002/11/10 13:53:47 hio Exp $
 * ----------------------------------------------------------------------------
 */

#include <windows.h>
#include "rishu_proxy.h"
#include <stdio.h>

/* ----------------------------------------------------------------------------
 * Request の初期化.
 */
RishuProxy_Request* RishuProxy_Request_init(RishuProxy_Request* request,
			     WORD wSize,
			     WORD version)
{
  if( wSize<sizeof(RishuProxy_Request) )
  {
    return NULL;
  }
  request->wSize = wSize;
  request->wVersion = version;
  /* Path */
  request->path = NULL;
  request->dwPathLen = 0;
  /* 送信データ */
  request->data = NULL;
  request->dwDataLen = 0;
  request->dwDataSize = 0;
  return request;
}

/* ----------------------------------------------------------------------------
 * Request の破棄.
 */
void RishuProxy_Request_discard(RishuProxy_Request* request)
{
  if( request->path )
  {
    HeapFree(GetProcessHeap(),0,request->path);
    request->path = NULL;
    request->dwPathLen = 0;
  }
  if( request->data )
  {
    HeapFree(GetProcessHeap(),0,request->data);
    request->data = NULL;
    request->dwDataLen = 0;
    request->dwDataSize = 0;
  }
}

/* ----------------------------------------------------------------------------
 * path の設定.
 */
void RishuProxy_Request_setPath(RishuProxy_Request* request,
				LPCVOID path,
				DWORD len)
{
  if( !request->path )
  {
    request->path = HeapAlloc(GetProcessHeap(),0,len);
  }else
  {
    request->path = HeapReAlloc(GetProcessHeap(),0,request->path,len);
  }
  if( !request->path )
  {
    return;
  }
  CopyMemory(request->path,path,len);
  request->dwPathLen = len;
  return;
}

/* ----------------------------------------------------------------------------
 * 書き込み.
 */
void RishuProxy_Request_write(RishuProxy_Request* request,
			      LPCVOID data,
			      DWORD len)
{
  if( request->dwDataSize-request->dwDataLen<len )
  {
    if( !request->data )
    {
      request->data = HeapAlloc(GetProcessHeap(),0,len);
      if( !request->data )
      {
	return;
      }
    }else
    {
      register const HGLOBAL olddata = request->data;
      request->data = HeapReAlloc(GetProcessHeap(),0,olddata,request->dwDataLen+len);
      if( !request->data )
      {
	request->data = olddata;
	return;
      }
    }
    request->dwDataSize = HeapSize(GetProcessHeap(),0,request->data);
  }
  CopyMemory((LPBYTE)request->data+request->dwDataLen,data,len);
  request->dwDataLen   += len;
  return;
}

/* ----------------------------------------------------------------------------
 * 送信.
 */
BOOL RishuProxy_Request_send(RishuProxy_Request* request,
			     RishuProxy* proxy)
{
  BOOL res;
  DWORD written;
  char buf[512];
  int len;
  
  if( !request->data )
  {
    if( RishuProxy_warnlogIsEnabled(proxy) )
    {
      RishuProxy_log(proxy,TEXT("warn"),
		     TEXT("RishuProxy::Request::send"),
		     TEXT("empty request\n")
		     );
    }
    return FALSE;
  }
  
  len = _snprintf(buf,512,
		  "PROXY %.*s RISHU/%d.%d\r\n",
		  request->dwPathLen, request->path,
		  request->wVersion/0x100, request->wVersion&0x255
		  );
  if( len<0 )
  { /* -len が必要なバイト数. */
    /* fall back is not implemented yet */
    return FALSE;
  }
  res = WriteFile(RishuProxy_getChildInput(proxy),
		  buf,
		  len,
		  &written,
		  NULL
		  );
  if( !res )
  {
    if( RishuProxy_errorlogIsEnabled(proxy) )
    {
      LPTSTR cause = getLastErrorMessage();
      RishuProxy_log(proxy,TEXT("error"),
		     TEXT("RishuProxy::Reqeust::send"),
		     TEXT("WriteFile failed (1/2)\n>> %s\n")
		     );
      LocalFree(cause);
    }
    return FALSE;
  }
  
  res = WriteFile(RishuProxy_getChildInput(proxy),
		  request->data,
		  request->dwDataLen,
		  &written,
		  NULL
		  );
  if( !res )
  {
    if( RishuProxy_errorlogIsEnabled(proxy) )
    {
      LPTSTR cause = getLastErrorMessage();
      RishuProxy_log(proxy,TEXT("error"),
		     TEXT("RishuProxy::Reqeust::send"),
		     TEXT("WriteFile failed (2/2)\n>> %s\n")
		     );
      LocalFree(cause);
    }
    return FALSE;
  }
  
  res = FlushFileBuffers(RishuProxy_getChildInput(proxy));
  if( !res && RishuProxy_warnlogIsEnabled(proxy) )
  {
    LPTSTR cause = getLastErrorMessage();
    RishuProxy_log(proxy,TEXT("warn"),
		   TEXT("RishuProxy::Reqeust::send"),
		   TEXT("[FlushFileBuffers failed]\n"
			"[[\n%s]]\n"),
		   cause
		   );
    LocalFree(cause);
  }

  if( RishuProxy_tracelogIsEnabled(proxy) )
  {
    /* リクエスト行と送信データで２つ書き込み. */
    RishuProxy_log(proxy,TEXT("trace"),
		   TEXT("RishuProxy::Reqeust::send"),
		   TEXT("[send message]\n"
			"[[\n%.*s%.*s]]\n"),
		   len,buf,
		   request->dwDataLen, request->data
		   );
  }
#if 0
    /* 通信内容 */
    for( i=0; i<proxy->dwReadBufLen; ++i )
    {
      register const unsigned char c = ptr[i];
      if( 0x20<=c && c<=0x7e ) putchar(c);
      else if( c=='\r' ) putchar('\\'),putchar('r');
      else if( c=='\n' ) putchar('\\'),putchar('n');
      else printf("\\x%02x",c);
    }
    putchar('\n');
#endif
  return TRUE;
}
